@@ -1,5 +1,5 @@
 /* Definitions for kNetBSD-based GNU systems with ELF format
-   Copyright (C) 2004
+   Copyright (C) 2004, 2006, 2007, 2011
    Free Software Foundation, Inc.
    Contributed by Robert Millan.
 
@@ -7,7 +7,7 @@ This file is part of GCC.
 
 GCC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GCC is distributed in the hope that it will be useful,
@@ -16,12 +16,11 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GCC; see the file COPYING.  If not, write to
-the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.  */
+along with GCC; see the file COPYING3.  If not see
+<http://www.gnu.org/licenses/>.  */
 
-#undef LINUX_TARGET_OS_CPP_BUILTINS    
-#define LINUX_TARGET_OS_CPP_BUILTINS()		\
+#undef GNU_USER_TARGET_OS_CPP_BUILTINS    
+#define GNU_USER_TARGET_OS_CPP_BUILTINS()		\
   do						\
     {						\
 	builtin_define ("__NetBSD_kernel__");	\
@@ -32,5 +31,6 @@ Boston, MA 02110-1301, USA.  */
     }						\
   while (0)
 
-#undef DYNAMIC_LINKER
-#define DYNAMIC_LINKER "/lib/ld.so.1"
+
+#undef GNU_USER_DYNAMIC_LINKER
+#define GNU_USER_DYNAMIC_LINKER "/lib/ld.so.1"
