@@ -6,8 +6,9 @@
    Copyright (C) 2002 Free Software Foundation Inc.
    Contributed by Andreas Bauer <baueran@in.tum.de>  */
 
-/* { dg-do run { target i?86-*-* s390*-*-* x86_64-*-*} } */
-/* { dg-options "-O2 -foptimize-sibling-calls" } */
+/* { dg-do run { target i?86-*-* x86_64-*-* s390*-*-* } } */
+/* { dg-skip-if "" { { i?86-*-* x86_64-*-* } && { ia32 && { ! nonpic } } } { "*" } { "" } } */
+/* { dg-options "-O2 -foptimize-sibling-calls -fno-ipa-cp" } */
 
 extern void abort (void);
 extern void exit (int);
@@ -26,7 +27,7 @@ main ()
   exit (0);
 }
 
-int
+int __attribute__ ((noinline))
 bar (b)
      int b;
 {
@@ -36,7 +37,7 @@ bar (b)
     abort ();
 }
 
-int
+int __attribute__ ((noinline))
 foo (f)
      int f;
 {
