@@ -1,5 +1,5 @@
 /* CertReqCmd.java -- The certreq command handler of the keytool
-   Copyright (C) 2006 Free Software Foundation, Inc.
+   Copyright (C) 2006, 2007 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,7 +39,7 @@ exception statement from your version. */
 package gnu.classpath.tools.keytool;
 
 import gnu.classpath.Configuration;
-import gnu.classpath.tools.getopt.ClasspathToolParser;
+import gnu.classpath.tools.common.ClasspathToolParser;
 import gnu.classpath.tools.getopt.Option;
 import gnu.classpath.tools.getopt.OptionException;
 import gnu.classpath.tools.getopt.OptionGroup;
@@ -50,7 +50,7 @@ import gnu.java.security.der.DER;
 import gnu.java.security.der.DERReader;
 import gnu.java.security.der.DERValue;
 import gnu.java.security.der.DERWriter;
-import gnu.java.security.util.Base64;
+import gnu.java.util.Base64;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -84,14 +84,14 @@ import javax.security.auth.x500.X500Principal;
  *   signatureAlgorithm        AlgorithmIdentifier,
  *   signature                 BIT STRING
  * }
- * 
+ *
  * CertificationRequestInfo ::= SEQUENCE {
  *   version           INTEGER -- v1(0)
  *   subject           Name,
  *   subjectPKInfo     SubjectPublicKeyInfo,
  *   attributes    [0] IMPLICIT Attributes -- see note later
  * }
- * 
+ *
  * SubjectPublicKeyInfo ::= SEQUENCE {
  *   algorithm         AlgorithmIdentifier,
  *   subjectPublicKey  BIT STRING
@@ -114,7 +114,7 @@ import javax.security.auth.x500.X500Principal;
  *      a default value of <code>mykey</code> shall be used when this option is
  *      omitted from the command line.
  *      <p></dd>
- *      
+ *
  *      <dt>-sigalg ALGORITHM</dt>
  *      <dd>The canonical name of the digital signature algorithm to use for
  *      signing the certificate. If this option is omitted, a default value will
@@ -124,11 +124,11 @@ import javax.security.auth.x500.X500Principal;
  *      If on the other hand the private key is an <code>RSA</code> one, then
  *      the tool will use <code>MD5withRSA</code> as the signature algorithm.
  *      <p></dd>
- *      
+ *
  *      <dt>-file FILE_NAME</dt>
- *      
+ *
  *      <dt>-keypass PASSWORD</dt>
- *      
+ *
  *      <dt>-storetype STORE_TYPE</dt>
  *      <dd>Use this option to specify the type of the key store to use. The
  *      default value, if this option is omitted, is that of the property
@@ -136,7 +136,7 @@ import javax.security.auth.x500.X500Principal;
  *      obtained by invoking the {@link java.security.KeyStore#getDefaultType()}
  *      static method.
  *      <p></dd>
- *      
+ *
  *      <dt>-keystore URL</dt>
  *      <dd>Use this option to specify the location of the key store to use.
  *      The default value is a file {@link java.net.URL} referencing the file
@@ -149,13 +149,13 @@ import javax.security.auth.x500.X500Principal;
  *      name (with absolute or relative path-name) of a key store --as if the
  *      protocol was <code>file:</code>.
  *      <p></dd>
- *      
+ *
  *      <dt>-storepass PASSWORD</dt>
  *      <dd>Use this option to specify the password protecting the key store. If
  *      this option is omitted from the command line, you will be prompted to
  *      provide a password.
  *      <p></dd>
- *      
+ *
  *      <dt>-provider PROVIDER_CLASS_NAME</dt>
  *      <dd>A fully qualified class name of a Security Provider to add to the
  *      current list of Security Providers already installed in the JVM in-use.
@@ -163,11 +163,11 @@ import javax.security.auth.x500.X500Principal;
  *      added to the runtime --i.e. it was not already installed-- then the tool
  *      will attempt to removed this Security Provider before exiting.
  *      <p></dd>
- *      
+ *
  *      <dt>-v</dt>
  *      <dd>Use this option to enable more verbose output.
  *      <p></dd>
- *      
+ *
  *      <dt>-attributes</dt>
  *      <dd>Use this option to force the tool to encode a NULL DER value in the
  *      CSR as the value of the Attributes field.</dd>
@@ -293,7 +293,7 @@ class CertReqCmd extends Command
     byte[] derBytes = getCSR(aliasName, publicKey, (PrivateKey) privateKey);
 
     // 4. encode it in base-64 and write it to outStream
-    String encoded = Base64.encode(derBytes, 0, derBytes.length, true);
+    String encoded = Base64.encode(derBytes, 72);
     PrintWriter writer = new PrintWriter(outStream, true);
     writer.println("-----BEGIN NEW CERTIFICATE REQUEST-----"); //$NON-NLS-1$
     writer.println(encoded);
@@ -421,9 +421,9 @@ class CertReqCmd extends Command
    * @param publicKey
    * @param privateKey
    * @return the DER encoded Certificate Signing Request.
-   * @throws IOException 
-   * @throws InvalidKeyException 
-   * @throws SignatureException 
+   * @throws IOException
+   * @throws InvalidKeyException
+   * @throws SignatureException
    */
   private byte[] getCSR(X500Principal aliasName, PublicKey publicKey,
                         PrivateKey privateKey)
@@ -454,7 +454,7 @@ class CertReqCmd extends Command
     sigAlgorithm.trimToSize();
     DERValue derSignatureAlgorithm = new DERValue(DER.CONSTRUCTED | DER.SEQUENCE,
                                                   sigAlgorithm);
-    
+
     signatureAlgorithm.initSign(privateKey);
     signatureAlgorithm.update(derCertRequestInfo.getEncoded());
     byte[] sigBytes = signatureAlgorithm.sign();
