@@ -1,6 +1,7 @@
 // Locale support -*- C++ -*-
 
-// Copyright (C) 1997, 1998, 1999, 2003, 2009 Free Software Foundation, Inc.
+// Copyright (C) 1997, 1998, 1999, 2003, 2009, 2010
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,10 +26,12 @@
 //
 // ISO C++ 14882: 22.1  Locales
 //
-  
+
 // Default information, may not be appropriate for specific host.
-  
-_GLIBCXX_BEGIN_NAMESPACE(std)
+
+namespace std _GLIBCXX_VISIBILITY(default)
+{
+_GLIBCXX_BEGIN_NAMESPACE_VERSION
 
   /// @brief  Base class for ctype.
   struct ctype_base
@@ -38,7 +41,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 
     // NB: Offsets into ctype<char>::_M_table force a particular size
     // on the mask type. Because of this, we don't use an enum.
-    typedef unsigned int 	mask;   
+    typedef unsigned int 	mask;
     static const mask upper    	= 1 << 0;
     static const mask lower 	= 1 << 1;
     static const mask alpha 	= 1 << 2;
@@ -46,10 +49,11 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
     static const mask xdigit 	= 1 << 4;
     static const mask space 	= 1 << 5;
     static const mask print 	= 1 << 6;
-    static const mask graph 	= (1 << 2) | (1 << 3) | (1 << 9);  // alnum|punct
+    static const mask graph 	= (1 << 2) | (1 << 3) | (1 << 9); // alnum|punct
     static const mask cntrl 	= 1 << 8;
     static const mask punct 	= 1 << 9;
     static const mask alnum 	= (1 << 2) | (1 << 3);  // alpha|digit
   };
 
-_GLIBCXX_END_NAMESPACE
+_GLIBCXX_END_NAMESPACE_VERSION
+} // namespace
