@@ -1,6 +1,6 @@
 // 2007-04-27  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2007, 2008, 2009 Free Software Foundation
+// Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,8 +18,7 @@
 // <http://www.gnu.org/licenses/>.
 
 // { dg-do compile }
-// { dg-error "no matching" "" { target *-*-* } 1383 }
-// { dg-excess-errors "" }
+// { dg-error "no matching" "" { target *-*-* } 1501 }
 
 #include <list>
 
@@ -30,6 +29,7 @@ struct A
 
 void f()
 {
-  std::list<A> l;
+  typedef std::list<A> list_type;
+  list_type l;
   l.assign(10, 1);
 }
