@@ -22,30 +22,32 @@
 #include <fstream>
 #include <testsuite_hooks.h>
 
-const char name_03[] = "tmp_11389-3";
+const char name_04[] = "tmp_11389-4";
 
-void test03()
+void test04()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
 
   filebuf fbout;
-  fbout.open(name_03, ios_base::out);
+  fbout.open(name_04, ios_base::out);
   fbout.sputc('a');
+  fbout.sputc('b');
   fbout.close();
   
   wfilebuf fbin;
   locale loc(locale("en_US.UTF-8"));
   fbin.pubimbue(loc);
   fbin.pubsetbuf(0, 0);
-  fbin.open(name_03, ios_base::in);
+  fbin.open(name_04, ios_base::in);
   VERIFY( fbin.sbumpc() == L'a' );
+  VERIFY( fbin.sbumpc() == L'b' );
   VERIFY( fbin.sgetc() == wfilebuf::traits_type::eof() );
   fbin.close();
 }
 
 int main()
 {
-  test03();
+  test04();
   return 0;
 }
