@@ -30,10 +30,10 @@ template<typename T>
 
 int main()
 {
-  check_remove_unsigned<bool>();  // { dg-error "instantiated from" }
-  check_remove_unsigned<wchar_t>();  // { dg-error "instantiated from" }
+  check_remove_unsigned<bool>();  // { dg-error "required from" }
+  check_remove_unsigned<wchar_t>();  // { dg-error "required from" }
   return 0;
 }
 
 // { dg-error "invalid use of incomplete" "" { target *-*-* } 28 } 
-// { dg-error "declaration of" "" { target *-*-* } 107 }
+// { dg-error "declaration of" "" { target *-*-* } 106 }
